;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#xe8c44e3314c8394941f43133103f374f . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x412189623fd6d1ba229aef2852a83399 . "./code.cg2js/cg-printing.fasl")
(#x236816ea53371125dfcb5211b7a0b91c . "./code.cg2js/cg-caret.fasl")
(#x65071f77095a7397f5e34d2633e5c64b . "./code.cg2js/cg-item-list.fasl")
(#xf0d00e95c906d656204c9ba12f4618fb . "./code.cg2js/cg-progress-indicator.fasl")
(#xbdd29aa11e02296fe8bff36b0a74c0ef . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#x9f91f2ac9dfe719d9bc7d99c8ffed162 . "./code.cg2js/cg-outline.fasl")
(#x94f3dd9c6713a631b7ae362dcf97e52c . "./code.cg2js/cg-palette.fasl")
(#x76786d98b571bad950f0b814e0889b2a . "./code.cg2js/cg-selected-object.fasl")
(#xa00a8fbb567bbed3e734086a76fb4118 . "./code.cg2js/cg-status-bar.fasl")
(#x882616ca7313957af979bd1bcd94859a . "./code.cg2js/cg-wrap-string.fasl")
(#xc568ef6e07a96e184a2b1a1e6e8b5df5 . "./code.cg2js/cg-fill-texture.fasl")
(#xceeb1efc60ad10f526fc84fb41d6e0b7 . "./code.cg2js/cg-layout.fasl")
(#x649ab484d9992ee1c820c734516425fe . "./code.cg2js/cg-timer.fasl")
(#xe6111132838fe5fa5f10cefdbf467313 . "./code.cg2js/cg-dialog-item.fasl")
(#x97c66b310791d6c4d9956a65359511e1 . "./code.cg2js/cg-button.fasl")
(#x6c484dc7aa51ab2123a887e1ebce33a9 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#x7a077ff9b0a103a98c450157b787515c . "./code.cg2js/cg-tray.fasl")
(#x3c215f88f215e18a5236f0f247384345 . "./code.cg2js/cg-plot-widget.fasl")
(#x5a17876f171b687c98a9c79dd95f1f00 . "./code.cg2js/cg-pixmap.fasl")
(#xb39e4b749e95b5104007eeaa4a873831 . "./code.cg2js/cg-font-dialog.fasl")
(#x704a9bc2352b666569e9e83b13f3d37f . "./code.cg2js/cg-os-widget.fasl")
(#xe8437f7375530ebb8824b6a6272ccb8b . "./code.cg2js/cg-text-or-combo.fasl")
(#x88b119b79b9efce6999d01dcc14d2f82 . "./code.cg2js/cg-trackbar.fasl")
(#xd5907c8776f308c4a64ab26d27d7580f . "./code.cg2js/cg-up-down-control.fasl")
(#x65de09738b653ea6ec7511526946de75 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#xad61fc4425b3a6d6093ca060855cf9d2 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x718add712d33b15c4b313c1266c88625 . "./code.cg2js/cg-intersect-posbox.fasl")
(#xa85b466037d241e11521407a6ed45e98 . "./code.cg2js/cg-bitmap-stream.fasl")
(#xedb98762eb9660c41e682a8eefe4eb2e . "./code.cg2js/cg-chart-widget.fasl")
(#xfbd965c3f35dda47e62fb045a8141e5d . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x7ece170233900112a2391dabcef84c4d . "./code.cg2js/cg-group-box.fasl")
(#xca2fb4188b117e714ac0e03b3a317bfb . "./code.cg2js/cg-static-text.fasl")
(#x2c6dc0d7c727254923ecc549e3c3ec13 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x503f472cdf037223b9942141cadf18e9 . "./code.cg2js/cg-color-dialog.fasl")
(#xc030b4f2dd6688d2ff7bd5037b703768 . "./code.cg2js/cg-intersect.fasl")
(#x57122b8d386f865308b673556140f9b8 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x1508113b982d54573a1a8f2c730eabe9 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#x52aa42da983450063ba1b13c0bab8011 . "./code.cg2js/cg-html-dialog.fasl")
(#x04aa529db3233d313023c2b128aab339 . "./code.cg2js/cg-lettered-menu.fasl")
(#x50b0f20bad0ec29e98e90612930b9c2b . "./code.cg2js/cg-class-grid.fasl")
(#x572817de442277e5ee61907b20e3b28f . "./code.cg2js/cg-pixmap-printing.fasl")
(#xb98052f57b5205a95be39228a2ae623d . "./code.cg2js/cg-calendar.fasl")
(#x8d15c52442164feece61ff0d1e0aced3 . "./code.cg2js/cg-editable-text.fasl")
(#xf16ebda0b5c972887f8de611bf295f3e . "./code.cg2js/cg-property.fasl")
(#x0bb52ca558fb91a7613eb45b29f48e73 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#xb815fead851080304fd983e63c032714 . "./code.cg2js/cg-grid-widget.fasl")
(#xafbe61e4700ce6a51b77bcc9de3bc953 . "./code.cg2js/cg-paren-matching.fasl")
(#xc9e95db07e97cb28d8a540c3e3ffba3e . "./code.cg2js/cg-dev.fasl")
(#x3b43bb76c4000dde81dade5bcd7ee152 . "./code.cg2js/cg-drawable.fasl")
(#xcc0be3a5a4c1c67145ea159f3c14e0ca . "./code.cg2js/cg-curve.fasl")
(#x6829aeb985286e8c089b36396a5ce157 . "./code.cg2js/cg-lisp-widget.fasl")
(#x655e8fd495d980fc7dfaea79537ef3dd . "./code.cg2js/cg-dde-utils.fasl")
(#x02e6cd47f7e49517ca0bfae4ba2ea938 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xdb14a690e7ce6028923e198ac85a3e4f . "./code.cg2js/cg-object-editor-layout.fasl")
(#xae6ea104f00dc62674908f686b2b7331 . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xaf09232e6c645053e3e79235b1a2bedf . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#x75bbd76971a00a811b7606a7ddfaecd9 . "./code.cg2js/cg-lamp.fasl")
(#x3f6189d2a18f6934ff6ff17f9d971576 . "./code.cg2js/cg-common-control.fasl")
(#xc997e99eb8efd04bf1b4138b3bba6949 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#x4697e7b81c9de9612177ff66b21a00f2 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x5856365c01dae4d2dcbc29cae998a0f2 . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#x24a54bdf10961ecbf23d36b4412e32bd . "./code.cg2js/cg-project-window.fasl")
(#x44a9563c1f1160bda1b537912de67a48 . "./code.cg2js/cg-sample-file-menu.fasl")
(#xdf177f8fc0d83c664884513996e1a367 . "./code.cg2js/cg-class-support.fasl")
(#xc92405634c7b1655ca1f003a98a532b2 . "./code.cg2js/cg-multi-picture-button.fasl")
(#x7eee2d85cbf6dcc8ddf7a1506cb7a010 . "./code.cg2js/cg-yes-no-list.fasl")
(#x8b8e9bbbb4156f4311b64acda611abab . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xd7b3ef9035fe249f2142f9deb61d88f6 . "./code.cg2js/cg-string-dialog.fasl")
(#x6884d2f06542923e4a958aff52d478b2 . "./code.cg2js/cg-menu-tooltip.fasl")
(#x6cee4f1de1316ec22705331ca3da97e3 . "./code.cg2js/cg-dropping-outline.fasl")
(#xeb55c5e19dd85ad90bdc65dbe403a37f . "./code.cg2js/cg.fasl")
(#x8c30b85dde8cbaec1b0f129fad10de6e . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x334b7bd3bab04657b83a4c0a0357efb3 . "./code.cg2js/cg-scaling-stream.fasl")
(#x4ff9d56c97d49dd8a118358665af1071 . "./code.cg2js/cg-yes-no-string.fasl")
(#xa49db2bf2f619bce6297a56baf11f9b2 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x7c62dbd5f835e5b88c0856a1fac52566 . "./code.cg2js/cg-web-browser.fasl")
(#x08e70f80b6d59c7ee6f3336dc8b52e35 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x724cb8e4bf11d5575da902a26a9cf43a . "./code.cg2js/cg-drag-and-drop.fasl")
(#x0bd8f725eb5b724f24ff77fb6de255d8 . "./code.cg2js/cg-message-dialog.fasl")
(#x73aadbeca1aa10824a97b3a621331c70 . "./code.cg2js/cg-comtab.fasl")
(#x1c4e7f87137e10846f1865b670c84dd7 . "./code.cg2js/cg-base.fasl")
(#xb591e336c20043e07c5f930b121cc960 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x1474f225a30cf7013972640027b0f407 . "./code.cg2js/cg-text-edit-pane.fasl")
(#xa3097993ca2c78cfc97d4b21ab9c7e32 . "./code.cg2js/cg-utility-dialog.fasl")
(#x75053974478db5c8e196fa01fa216a17 . "./code.cg2js/cg-object-editor.fasl")
(#xd5bd3043a537baaf41eec7833e406f82 . "./code.cg2js/cg-edit-in-place.fasl")
(#x52bb476c58786ea326778f6e6174ee60 . "./code.cg2js/cg-radio-button.fasl")
(#xdc5de116920ef5c627b3f56d895893e2 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#x383083631ea4f43e071efd902774008e . "./code.cg2js/cg-nodes.fasl")
(#x2847d07f94b0298b2da8d17a868ea6c6 . "./code.cg2js/cg-get-position.fasl")
(#xef132ea2ffcd0298d55cb906831f5644 . "./code.cg2js/cg-directory-dialog.fasl")
(#x9e23033c08de5015cf788c52fab1939a . "./code.cg2js/cg-lisp-text.fasl")
(#xbf4512c54400043cb88ce8dda54e0c71 . "./code.cg2js/cg-combo-box.fasl")
(#xa1c82e5f1cabcac4c155f59c75c30503 . "./code.cg2js/cg-text-widget.fasl")
(#x7de603e17e374c845131a2aceec01676 . "./code.cg2js/cg-list-view.fasl")
(#x81ae7cd063e563b1647655da00dfa451 . "./code.cg2js/cg-graphics-context.fasl")
(#x66e0f46ff6e47dc01b38f4b6aa7ba5dc . "./code.cg2js/cg-get-pixmap.fasl")
(#x108a9c726e83974445446754b2362fd4 . "./code.cg2js/cg-menu.fasl")
(#x814f0bd5eb84d424897a4c1c8d036369 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x343eeb6a7dc457d7a6fed2a93acecc90 . "./code.cg2js/cg-pixmap-rotate.fasl")
(#x511966c67c339d7dab0c8c81005be052 . "./code.cg2js/cg-tooltip.fasl")
(#x4df331c8a27c752b9bd9dd588c871014 . "./code.cg2js/cg-html-widget.fasl")
(#x23504a46a0a238b6c6e06099c6dd64f5 . "./code.cg2js/cg-choice-list.fasl")
(#xb088bcd49f1bc1d7efc28a4129cbdf24 . "./code.cg2js/cg-icon.fasl")
(#x48d7300e0b29f6ca36d265a06047e710 . "./code.cg2js/cg-acache.fasl")
(#x7ee6eda72067edf575e98da1d1c66d81 . "./code.cg2js/cg-template-string.fasl")
(#xf81afded4c7db616c94584aa58a3ad70 . "./code.cg2js/cg-bitmap-pane.fasl")
(#xc52f86c08e6fb96ccfbc6d1f0b7e242b . "./code.cg2js/cg-toggling-widget.fasl")
(#xa6d3010289938235633d34466d3260ba . "./code.cg2js/cg-tab-control.fasl")
(#xeca85d43a7a26b0e02bcdcce6e342be3 . "./code.cg2js/cg-split-bar.fasl")
(#xfc3ef3a063e0342b6746792689a7cc71 . "./code.cg2js/cg-picture-widget.fasl")
(#x26d23e45593b387488c9059bb43330b4 . "./code.cg2js/cg-player.fasl")
(#xc6563f1938662747120b8f662bf5565d . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#x3ac40cc93d487ce8b77fc129d60b28e1 . "./code.cg2js/cg-choose-printer.fasl")
(#x6ac9c5ac019847a5b68e4f95d892e291 . "./code.cg2js/cg-os-window.fasl")
(#xcfa7ffdcef276db2f9e5dd16b8b86bc4 . "./code.cg2js/cg-scroll-bar.fasl")
(#xc053d2948eaf2b815f8f7ef84fdf9c52 . "./code.cg2js/cg-clipboard.fasl")
(#xe301768a6d669f280a1e99e5da85a58f . "./code.cg2js/cg-find-string-dialog.fasl")
(#xc4ce01e124bd40cd9e4186f63bc689ff . "./code.cg2js/cg-hotspot.fasl")
(#x2741fcde5fe2816fc4ba56dee1f01175 . "./code.cg2js/cg-check-box.fasl")
(#x5039ad23bab74cbfb26f8b0b38468287 . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x5963d76f89bc92c6cdb5289eb58d77e5 . "./code.cg2js/cg-file-dialog.fasl")
(#x06db793cb90157210e3cb4384ea09c75 . "./code.cg2js/cg-toolbar.fasl")
(#x44be100924848ac97aea60a0ceda9414 . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x76f3fbafaa8cdc6f82e96079ac0e7058 . "./code.cg2js/cg-chart-or-plot.fasl")
(#x2ec8b6ab62dc4ec14f68cfdee7deaef9 . "./code.cg2js/cg-shortcut-menu.fasl")
(#xbe10b964521fd0a4d63aac0ce28ceec5 . "./code.idejs/ide-project.fasl")
(#x75890158c25c5b953c75111fb7902edf . "./code.idejs/ide-intro.fasl")
(#x7490ba95051546b15dc143071fc6cc3f . "./code.idejs/ide-find-in-files.fasl")
(#xd54fff86428afa4415f1269a5f32cce9 . "./code.idejs/ide-grapher.fasl")
(#xa0d2f770507c69c1905a5eb746375d4d . "./code.idejs/ide-inspector.fasl")
(#x969cebce0ddd5a216f7acebf5fa56cfa . "./code.idejs/ide-editor.fasl")
(#x2e272f6170ac0f63c4a8010bfbed3077 . "./code.idejs/ide-window-list.fasl")
(#x53699eb35db44b95d497dfba84ec26dd . "./code.idejs/ide-process-browser.fasl")
(#xdf64a763def62719b0286a0d2bfec6d9 . "./code.idejs/ide-patch-dialog.fasl")
(#x4a4ee5fa8b41ac443f68e4a2926629ad . "./code.idejs/ide-class-grapher.fasl")
(#x7e5954bfac60599fc1e63a6b19e4ceb4 . "./code.idejs/ide-base.fasl")
(#x12ddb8fb446da36f93db4cbc95585a19 . "./code.idejs/ide.fasl")
(#x1633ee8771c1d69f14090894837da29a . "./code.idejs/ide-trace.fasl")
(#x0ab321f17ad8b4a01e0af0ee2ec2f07c . "./code.idejs/ide-form.fasl")
(#x48a8733322c86e2aed005aa54e55fac7 . "./code.idejs/ide-find-definitions.fasl")
(#xbcd8310dfaa1794654ce5716edc98afb . "./code.idejs/ide-apropos.fasl")
(#x2bebf9ffcb4134ce0aaa0d3decebc3b3 . "./code.idejs/ide-class-browser.fasl")
(#xa2eefcb30dc8b1c3f7d3f7e845090afe . "./code.idejs/ide-navigator.fasl")
(#xf32ab896c5fad833668e3ee8bab9a9d1 . "./code.idejs/ide-clipboard.fasl")
(#x22ff85b8d1b9d00dc11d6aa1187ceb3d . "./code.idejs/ide-acache.fasl")
(#xfd5a4cac8bcef8e6fdc673467c9b42d3 . "./code.idejs/ide-keyshort.fasl")
(#xd00f36453d5d828aef0cac12cce27165 . "./code.idejs/ide-package-list.fasl")
(#x74307d1b780b2da3914ffc5ea3609d8a . "./code.idejs/ide-dev.fasl")
(#x9a8f50ffdba20a10156ef75920aa924b . "./cgjs.html")
(#xdac1739373835d1a63cbafa2c3e0b06a . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (format t "Incorrect checksum on file: ~a" (cdr checksum))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js" "code.idejs"))
    (format s "
(let ((path (make-pathname
             :host \"sys\"
             :directory '(:relative \"~a\")
	     :type \"fasl\"
	     )))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows
  (let ((noninteractive (sys:getenv "ACL_NONINTERACTIVE")))
    (when (or (null noninteractive)
	      (string= "" noninteractive))
      (excl::internal-message-box msg "CG/JS Installation" :type :ok))))
